#! /usr/bin/perl
#
# Scan all subdirectories and create the apropriate index list.
# Scan the arguments and build a proper website from the input.
#
# $Header: /cvsroot/ac-archive/ac-archive-5/gendocs.pm,v 1.4 2005/01/17 06:52:49 guidod Exp $
#
package GenDocs;
use IO::Handle;

use strict;
my %o; # the options hash
my %out; # use as $out{$file}

# --------------------------------------------------------------------------
# some options are global - set the defaults here

my $htm = "html";
$htm= "htm" if $ARGV[0] eq "htm";
my $top_builddir=".";
my $topx="macros.html";
my $n = "\n"; my $br = "<br />\n";

my %style;
$style{P}{mark} = 'p class="para"';
$style{P}{head} = '.para { text-align : justify ; '.$n.
    '                      margin-right: 1em ;'.$n.
    '                      margin-left: 1em ; }';
$style{DT}{mark} = 'dt class="DT"';
$style{DD}{mark} = 'dd class="DD"';
$style{DT}{head} = '.DT       { font-weight : bold ; }';
$style{DD}{head} = '.DD       { text-align : justify ; margin-right: 1em ; }';

sub expand_html_style # $text -> $text
{
    my ($T,$xxx) = @_;
    my $tag;
    for $tag (keys %style) {
	next if not $style{$tag}{mark};
	my $mark = $style{$tag}{mark};
	my $ends = $mark; $ends =~ s/ .*//;
	if ($T =~ s{<$tag>}{<$mark>}gs) { $style{$tag}{used} = "!"; }
	if ($T =~ s{</$tag>}{<$ends>}gs) {$style{$tag}{used} = "!"; }
	if ($T =~ s{<$tag\ />}{$mark}gs)  {$style{$tag}{used} = "!"; }
    }
    return $T;
}


# -------------------------------------------------------------- OPT SCAN ---
# scanning the options and file arguments:
# we differ the input file by its file extension, and derive 
# their output name right here. This is for "macro" and "docus" 
# output type, some other output types will be generated later 
# from information found in these. 
# input structures:
#      @ARGV - the argument list
# output structures
#      $o{$option} ... options found in argument list
#      $out{$file} ... the filename to be generated at last step
#      $out{$file}{orig} the original file name in ARGV
#      $out{$file}{text} the text content of that file
#      $out{$file}{type} either "docus" or "macro"

sub scan_docus_file # $sourcefile (*.htm)
{
    my ($F,$xxx) = @_; my $file = $F."l"; $file =~ s/^\w-//;
    open F, "<$F" or return;
    $out{$file}{body} = join ("",<F>);
    $out{$file}{orig} = $F;
    $out{$file}{type} = "docus";
    close F;
}

sub scan_macro_file # $sourcefile (*.m4) ; $category
{
    my ($F,$into,$xxx) = @_; my $file = $F; $file =~ s:[.]m4$:.$htm:;
    open F, "<$F" or return;
    $out{$file}{text} = join ("", getlines F);
    $out{$file}{orig} = $F;
    $F =~ s:[.]m4$::;
    $out{$file}{name} = $F;
    $out{$file}{type} = "macro";
    $out{$file}{into} = $into if $into;
    $out{$file}{category} = $into if $into;
    close F;
}

sub scan_extra_file # $sourcefile (*.txt) ; $category
{
    my ($F,$into,$xxx) = @_; my $file = $F; $file =~ s:[.]txt$:.$htm:;
    open F, "<$F" or return;
    $out{$file}{text} = join ("", getlines F);
    $out{$file}{orig} = $F;
    $F =~ s:[.]txt$::;
    $out{$file}{name} = $F;
    $out{$file}{type} = "extra";
    $out{$file}{into} = $into if $into;
    $out{$file}{category} = $into if $into;
    close F;
}

sub scan_extra_data # *packagename::DATA (scanning its __DATA__ section)
{
    my ($F,$xxx) = @_; my $file = $$F; $file =~ s/::.*//;
    $out{$file}{text} = join ("", getlines $F);
    $out{$file}{orig} = $F;
    $out{$file}{name} = $file;
    $out{$file}{type} = "extra";
    close F;
}

sub scan_option # $optionstring -> $option | $nullstring
{
    if (/^--?(\w+)=(.*)/) { $o{$1} = $2; return $1; }
    if (/^--?no-(\w+)$/) { $o{$1} = ""; return $1; }
    if (/^--?(\w+)$/) { $o{$1} = "*"; return $1; }
    return "";
}

# UNUSED, see gendocs.pl
sub argv_scan
{
    for (@ARGV)
    {
	next if GenDocs::scan_option($_);
	if (/[.]htm$/) { scan_docus_file($_); }
	elsif (/[.]m4$/) { scan_macro_file($_); }
	elsif (/[.]txt$/) { scan_extra_file($_); }
    }
}

#&argv_scan();

# -------------------------------------------------------- MACRO2HTML SUB ---
# the following functions have been borrowed from macro2html.pl

sub escape_html
{
    my $text = $_[0];
    $text =~ s,\&,\&amp\;,g;
    $text =~ s,\",\&quot\;,g;
    $text =~ s,\<,\&lt\;,g;
    $text =~ s,\>,\&gt\;,g;
    return $text;
}

sub escape_blanks
{
    my $text = $_[0];
    $text =~ s,\s,\&nbsp\;,g;
    return $text;
}

sub parse_macro
{
    my $file = $_[0];
    # These are the parts that a macro should contain.
    # my ($synopsis, $version, $author, $copyright, $license);
    # my ($documentation, $macro, $category);

    # Parse the macro's header for the documentation and the
    # keywords;
    # my $pat_comment = '^dnl[ \\t]*(.*)[ \\t]*$';
    # my $pat_verbatim_comment = '^dnl[ \\t][ \\t](.*)[ \\t]*$';
    # my $pat_synopsis = '^dnl[ \\t]+@synopsis[ \\t]+(.*)[ \\t]*$';
    # my $pat_version = '^dnl[ \\t]+@version[ \\t]+(.*)[ \\t]*$';
    # my $pat_author = '^dnl[ \\t]+@author[ \\t]+(.*)[ \\t]*$';
    
    # abbrevs: see autoconf@gnu.org 19.1.2003, here is a new one:
    # @* AX_CFLAGS_WHATEVER (synopsis)
    # @* AX_CXXFLAGS_WHATEVER (alternative)
    # @, Installed Packags, experimental
    # @; the author <is@home>
    # @! license info
    # @(c) copyright info
    # @$Id: gendocs.pm,v 1.4 2005/01/17 06:52:49 guidod Exp $   

    my $last = "";
    for (split (/^/, $out{$file}{text} ))
    {
	chomp; 
	
	if ($last eq "macro")
	{
	    $out{$file}{macro} .= escape_html($_)."\n";
	    next;
	}

	if (m,^dnl\s+\@(?:synopsis|[*])\s+(.*)\s*$,i) # $pat_synopsis
	{
	    $last = "synopsis";
	    $out{$file}{$last} .= $br if length $out{$file}{$last};
	    $out{$file}{$last} .= escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:author|[;])\s+(.*)\s*$,i) # $pat_author
	{
	    $last = "author";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:license|[!])\s+(.*)\s*$,i) # license
	{
	    $last = "license";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:copyright|[(][Cc][)])\s+(.*)\s*$,i) # license
	{
	    $last = "copyright";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@version\s+(.*)\s*$,i) # $pat_version
	{
	    $last = "version";
	    my  $line = $1;
	    if ($line =~ m,^\$[I][d]: [^ ]+ ([^ ]+) ([^ ]+),i)
	    {
		$out{$file}{$last} = escape_html($1)." (".escape_html($2).")";
	    }
	    elsif ($line =~ m,^\$[I][d]:(.*)\$(.*),i)
	    {
		$out{$file}{$last} = escape_html($1." ".$2);
	    }else{
		$out{$file}{$last} = escape_html($line);
	    }
	}
	elsif (m,^dnl\s+\@(?:category|[\,])\s+(.*)\s*$,i) # categories
	{
	    $out{$file}{category} = $1; # do not put thru..
	}
	elsif (m,^dnl\s+(obsoleted[:\s].*)\s*$,i) # obsoleted
	{
	    $out{$file}{obsoleted} = $1;
	    $out{$file}{documentation} .= $br if $last eq "text";
	    $out{$file}{documentation} .= "<b>".escape_html($1)."</b>";
	    $out{$file}{documentation} .= $br;
	    $last = "enum";
	}
	elsif (m,^dnl\s+([*-]\s.*)\s*$,i) # enumeration
	{
	    $out{$file}{documentation} .= $br if $last eq "text";
	    $out{$file}{documentation} .= escape_html($1);
	    $out{$file}{documentation} .= $br;
	    $last = "enum";
	}
	elsif (m,^dnl\s\s(.*)\s*$,i) # $pat_verbatim_comment
	{
	    $out{$file}{documentation} .= $br if $last eq "text";
	    $out{$file}{documentation} .= "<code>&nbsp;&nbsp;";
	    $out{$file}{documentation} .= escape_blanks(escape_html($1));
	    $out{$file}{documentation} .= "</code>".$br;
	    $last = "code";
	}
	elsif (m,^dnl\s*(.*)\s*$,i) # $pat_comment
	{
	    if (not length $1) {
		$out{$file}{documentation} .= "<p />";
		$last = "para";
	    }else{
		$out{$file}{documentation} .= escape_html($1)."\n";
		$last = "text";
	    }
	}else{
	    $out{$file}{macro} = escape_html($_)."\n"; # print STDERR ".";
	    # The header part is over. Now the remainder of the file is the
	    # macro source code.
	    $last = "macro";
	}
    }
}

# ------------------------------------- ( Make up the formatted HTML page )
# called for each $file, -> generates $out{$file}{body}.
# the header parts have been already set in parse_macro,
#
# the header parts and body part will be combined into
# the full html output text - originally the full html
# text was made up right here (in $out{$file}{html}) but 
# now we delay the process to the final output loop. IF
# you set the $out{$file}{html} part somewhere in the next
# processing steps THEN the final output loop will prefer
# that one ELSE it creates the output file depending on
# its type merging the $out{$file}{body} part in the middle.
sub create_html                        
{
    my ($file,$xxx) = @_;

    $out{$file}{archive} = "";
    $out{$file}{archive} = " &nbsp; &nbsp; <small>$o{vers}</small>"
	if length                                 $o{vers};

    my $name = $out{$file}{orig}; $name =~ s:.*/::; $name =~ s:[.]\w+$::;
    my $body = "body";

    $out{$file}{title} = "Macro: ".$name;

    my $_width100=' width="100%"';
    my $_width50=' width="50%"';
    my $_center=' align="center"';
    my $O = '&nbsp;';
    my $T = "";

    if (length $name > 32)
    {
	$T .= "<table$_width100>".$n;
	$T .= "    <tr>".$n;
	$T .= "    <td$_width50$_center><a href=\"../\">"
	    ."Back to the Main Page.</a></td>".$n;
	$T .= "    <td$_width50$_center><a href=\"".$name.".m4\">"
	    ."Download the M4 Source.</a><!--DL--></td>".$n;
	$T .= "    </tr>".$n;
	$T .= "</table>".$n;
	$T .= "<hr /><p />".$n;
    
	$T .= "<h1 align=\"center\">".$name."</h1>".$n;
    }else{
	$T .= "<table$_width100>".$n;
	$T .= "    <tr>".$n;
	$T .= "    <td$_center><a href=\"../\">"
	    ."Back${O}to${O}the Main${O}Page.</a></td>".$n;

	$T .= "    <td$_center><h1>". $name."</h1></td>".$n;

	$T .= "    <td$_center><a href=\"".$name.".m4\">"
	    ."Download the${O}M4${O}Source.</a><!--DL--></td>".$n;
	$T .= "    </tr>".$n;
	$T .= "</table>".$n;
	$T .= "<hr /><p />".$n;
    }

    $T .= "<dl>\n";
    $T .= "<dt><big><b>Synopsis</b></big></dt><dd>".$n;
    $T .= " <p><code>".$out{$file}{synopsis}."</code></p>";
    $T .= $n."</dd>".$n;
    
    $T .= "<dt><big><b>Version</b></big></dt><dd>".$n;
    $T .= " <p>".$out{$file}{version};
    $T .= $n."".$out{$file}{archive};
    $T .= ' &nbsp; : &nbsp; <i>'.$out{$file}{category}."</i>"
	if length                                $out{$file}{category};
    $T .= $n."</p>";
    $T .= $n."</dd>".$n;
#   $T .= "<p><code>".$version."</code>".$n."</dd>";

    $T .= "<dt><big><b>Author</b></big></dt><dd>".$n;
    $T .= " <p>".$out{$file}{author};
    $T .= $n.$br." copyright: ".$out{$file}{copyright}
	if length                             $out{$file}{copyright};

    $T .= $n.$br." license: ".$out{$file}{license}
	if length                           $out{$file}{license};

    $T .= $n." </p>";
    $T .= $n."</dd>".$n;

    $T .= $n;
    $T .= "<dt><big><b>Description</b></big></dt><dd>".$n;
    $T .= " <div>\n".$out{$file}{documentation}."\n</div>";
    $T .= $n."</dd>".$n;

    $T .= $n.$n;
    $T .= "<dt><big><b>M4 Source Code</b></big></dt><dd>".$n;
    $T .= " <div><pre>\n".$out{$file}{macro}."\n</pre></div>";
    $T .= $n."</dd>".$n;
    $T .= "</dl>";

    $out{$file}{body} .= $T;
}

# UNUSED, see create_html above and generate_htmlbody below
# This is a heritage of macro2html which did create the html directly.
sub create_htmlbody
{
    my ($file,$xxx) = @_;
    &create_html($file);

    $out{$file}{html} = "";
    $out{$file}{html} .= "<html>".$n;
    $out{$file}{html} .= "<head>".$n;
    $out{$file}{html} .= "    <title>".$out{$file}{title}."</title>".$n;
    $out{$file}{html} .= "</head>".$n;
    $out{$file}{html} .= "<body>".$n;
    $out{$file}{html} .= $out{$file}{body};
    $out{$file}{html} .= "</body>".$n;
    $out{$file}{html} .= "</html>".$n;
}

# ------------------------------------------------------------- ASPELL SUB --
# we use an external tool gnu 'aspell' that will probably
# be present on most linux platforms atleast. Note how we
# use a perl pipe as input with the input file name which
# we take interest in to get indexed. The output of `aspell`
# is saved in $out{$file}{spelled} for later evaluation.
# This is just a routine, called from later parts.

sub parse_spell
{
    return if $o{nospell};
    my $file = $_[0];
    my $text = $out{$file}{documentation};
    return if not length $text;
    my %word;
    $text =~ s:\b[A-Z]+[_][A-Z_]*[A-Z]\b::g;
    $text =~ s:<code>[^<>]*</code>: :gs;
    $text =~ s:<[^<>]*>: :g;
    $text =~ s:\&lt\;:<:g;  $text =~ s:\&quot\;:\":g;  
    $text =~ s:\&gt\;:<:g;  $text =~ s:\&amp\;:\&:g;  
    $text =~ s:\&\w+\;: :g;
    $text =~ s{<(\w[^<>]*[.][^<>]+)>}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(\b\w+[/][\w/]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-l)(\w+\b)}{ $word{"lib".$2} = $1.$2; "  "}gmex;
    $text =~ s{(-L)(\w+\b)}{ $word{"lib-".$2} = $1.$2; "  "}gmex;
    $text =~ s{(--\w[\w-]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-?\w+-[\w-]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-[A-Z])(\w+\b)}{ $word{$1.$2} = $1.$2; "  "}gmex;
    $text =~ s:\=\s?\w+\b: :g;

    my $SPELL = "aspell --language-tag=english list";
    return if not open SPELL, "| $SPELL > $file.spell"; # <--- a pipe!!
    print SPELL $text; 
    close SPELL;
    return if not open SPELL, ">> $file.spell";
    for $text (keys %word) { print SPELL $text,$n; }
    close SPELL;
    $out{$file}{spelled} = "$file.spell";
#    $text = $file; $text =~ s:.*/::;
#    my $body; for $body (qw/html body/) 
#    {	if (exists $out{$file}{$body}) 
#	{   $out{$file}{$body} =~ s{<!--DL-->}{
#		"<!--DL-->".$n."<a href='$text.spell'>*</a>" }ex; 
#	} 
#    }
}

sub parse_spell_check
{
    return if $o{nospell};
    my $ver = `aspell --version`;
    if ($ver !~ /\d+[.]\d+/) {
	print STDERR "no spell found, $ver";
	$o{nospell} = "*";
    }
}

# -------------------------------------------------------------- DOC SCAN ---
# This is overly complicated and should be changed - It is about
# the "left navigation bar" which is broken into different sections
# each with a set of entries, and these sections/entries do not need
# to follow alphabetic order. Information about that is automatically
# derived from the input files but experiences with other projects
# have shown that it breaks more often than it helps to distribute
# the pieces into each input file. Instead, use a central "nav bar"
# template file that the various docs can attach to as linkends.

sub parse_docus
{
    my $file;
    for $file (keys %out)
    {
	if ($out{$file}{body} =~ /<!--\s*SORTING\s*\"([^\"]+)\"/)
	{
	    $out{$file}{sort} = $1 if not length $out{$file}{sort};
	}
	if ($out{$file}{body} =~ /<!--\s*CATEGORY\s*\"([^\"]+)\"/)
	{
	    $out{$file}{category} = $1 if not length $out{$file}{category};
	    $out{$file}{sort} = $1 if not length $out{$file}{sort};
	}
	if ($out{$file}{body} =~ /<!--\s*CHAPTER\s*\"([^\"]+)\"/)
	{
	    $out{$file}{chapter} = $1 if not length $out{$file}{chapter};
	}
	if ($out{$file}{body} =~ /<!--\s*SHORTNAME\s*\"([^\"]+)\"/)
	{
	    $out{$file}{title} = $1 if not length $out{$file}{title};
	}
    }
}

#&parse_docus();

# -------------------------------------------------------------- SCAN DIRS --
#
# For each category, find the .m4 macro files in it and list them.
#

sub scan_subdirs
{
    my ($topdir,$xxx) = @_;
    $topdir = $top_builddir if not $topdir;

    $out{$topx}{title} = "The Autoconf Macro Archive!";
    my $m4 = "m4";
    my $name;

    opendir(DIR, $topdir) || die("Can't open directory '$topdir': $!");
    while($name = readdir(DIR))          # open the base directory and
    {                                    # look for subdirectories.
	next if $name =~ /^(\.|CVS)/;    # and...
	if (-d $name)                    # ... scan ONLY these.
	{
	    next if -f "$name/Makefile.am";   # specials*/
	    next if -f "$name/acinclude.htm"; # doc/
	    next if -f "$name/requests";      # autom4te.cache/

	    my $file = $name."/index.html";
	    $out{$file}{name} = $name;
	    $out{$file}{type} = "index";
	    $out{$file}{file} = $name."/index.html";
	    $out{$file}{sort} = "ac-macros";

	    my $N = $name;
	    $N =~ s/_/ /g;
	    if ($N =~ /^[a-z]\w*$/)
	    {
		$out{$file}{sect} = "user";
		$out{$file}{into} = $name;
		$out{$file}{title} = $N."'s macros";
		$out{$file}{chapter} = '<a href="http://sourceforge.net/users/'
		    .$N.'">'.$N."</a>'s macros";
	    }else{
		$out{$file}{sect} = "gnus";
		$out{$file}{sect} = "gnu" if $name =~ /Support/;
		$out{$file}{into} = $name;
		$out{$file}{title} = $N;
		$out{$file}{chapter} = $N;
	    }
	    
	    my @files = <$name/*.$m4>; 
	    foreach $file (@files) 
	    {
		next if $file !~ /[.]$m4/;
		scan_macro_file ($file, $name);
	    }

	    @files = <$name/*.txt>;
	    foreach $file (@files)
	    {
		next if $file !~ /[.]txt/;
		scan_extra_file ($file, $name);
	    }
	}
    }
    closedir (DIR);
}

#&scan_subdirs($top_builddir);

# ---------------------------------------------------------- MACRO PAGES ---
# just before this, we ran a routine to autodect m4 files in
# subdirectories. We assume here that no more m4 files will be
# coming in, and we put them to parse* and create* subroutines.

sub gen_macro_pages
{
    &parse_spell_check ();

    my $file;
    for $file (keys %out) # foreach $file of type "macro"
    {
	next if not $out{$file}{type} eq "macro";
	print STDERR "." if not $o{quiet};
	parse_macro ($file) if not length $out{$file}{html}; # <-- MACRO2HTML
	create_html ($file) if not length $out{$file}{html}; # <-- MACRO2HTML
	parse_spell ($file) if not length $out{$file}{spelled};
    }   print STDERR $n if not $o{quiet};
}

#&gen_macro_pages();

# ------------------------------------------------------- EXTRA DEF PAGES ---
# per default this is stored in "gendocs.txt"

my %def;

$def{gnu}{webref} = "http://www.gnu.org/software/ac-archive/htmldoc/";
$def{gnu}{archive} = "GNU&nbsp;AC-Archive";
$def{gnus}{"C++_Support"}         = "index.html#CXX_SUPPORT";
$def{gnus}{"C_Support"}           = "index.html#C_SUPPORT";
$def{gnus}{"Cross_Compilation"}   = "index.html#CROSS_COMPILATION";
$def{gnus}{"Fortran_Support"}     = "index.html#FORTRAN_SUPPORT";
$def{gnus}{"Java_Support"}        = "index.html#JAVA_SUPPORT";
$def{gnus}{"Installed_Packages"}  = "index.html#INSTALLED_PACKAGES";
$def{gnus}{"Miscellaneous"}       = "index.html#MISC";
$def{html}{sfversion}="<!--VERSION-->*";

sub parse_extra_defs
{
    my $file;
    for $file (keys %out) # foreach $file of type "extra"
    {
	my $ini = "."; my $tion = ".";
	next if not $out{$file}{type} eq "extra";
	for (split (/^/, $out{$file}{text} ))
	{
	    chomp; 
	    if (/^\#/ or /^\s*$/)
	    {
		next;
	    }elsif (/^(\w+)\s*(.*)$/)
	    { 
		print STDERR $ini,'/',$tion,'="',$def{$ini}{$tion},'"',$n;
		$ini = lc($1); $tion = $2;
		$tion =~ s/\s*$//;
		$def{$ini}{$tion} = "";
	    }else
	    { 
		$def{$ini}{$tion} .= "\n " if length $def{$ini}{$tion};
		s/^\s+//; s/\s+$//; s/~/\&nbsp\;/g; s:<//>:<br />:g;
		$def{$ini}{$tion} .= $_; 
	    }
		print STDERR $ini,'/',$tion,'="',$def{$ini}{$tion},'"',$n
		    if $ini ne ".";
	}       print STDERR "------------------------",$n;
    }
}

sub fixup_extra_defs
{
    delete $def{"."};
    
    $def{html}{logo} = $def{html}{sfnetimage};
    $def{html}{sfversion}="<!--VERSION--><big><b>".$o{vers}."</b></big>" 
	if length $o{vers};
}

#&parse_extra_defs();
#&fixup_extra_defs();


# ----------------------------------------------------------- SEARCH INDEX ---
# create an inverted index for all words in the input. Note
# that we trim down the output of `aspell` here that reached
# us through "sub parse_aspell" while reading the original
# text directly on input (it was saved in $out{$file}{spelled}).

my %search;

sub search_index_words
{
    my $file;
    for $file (keys %out)
    {
	next if not length $out{$file}{spelled};
	my $F = $out{$file}{spelled};
	next if not open F, "< $F";            # read aspell results !!
	while (<F>)
	{
	    chomp;
	    next if not length;
	    next if not /^(\w|-)/;
	    next if /^[0-9]/;
	    next if /^test\w/;
	    next if /^(my|foo)-?(lib|pkg|config)/;
	    next if /^\w*(dir|file|level|LEVEL)$/;
	    next if /\b(CFLAGS|CXXFLAGS|LDFLAGS|CPPFLAGS|CVS|ACTION-IF)\b/;
	    next if /^(linux|autoconf|automake|arg|func|ie|htmlhmm|fs|faq)$/i;
	    next if /^(ac-archive|args|vm|todo|somedef|ahhm|subst|pc)$/i;
	    next if /^(os|okay|nocheck|filesys|filesystem|cflags|bugfix)$/i;
	    next if /^(autoheader|autoscan|api|a-z|a-z_|eg)$/i;
	    next if m:^(w/o|\w+\'s)$:;

	    $out{$file}{words} .= " " if length $out{$file}{words};
	    $out{$file}{words} .= $_;
	    $search{$_}{$file} = "";
	}
	close F;
    }    
}

sub search_index_body # $file
{
    my ($list,$xxx) = @_;
    # $index{$list}{file} = "$list";
    # $index{$list}{sect} = "virt";
    $out{$list}{file} = "$list";
    $out{$list}{name} = "search";
    $out{$list}{type} = "index";
    $out{$list}{sect} = "virt";
    $out{$list}{navi} = "search";
    $out{$list}{sort} = "ac-macros";
    $out{$list}{title} = "search index";
    $out{$list}{chapter} = "Inverted Search Index";
    $out{$list}{body} = "<dl>".$n;
    $out{$list}{info} = "";
    my $name;
    for $name (sort keys %search)
    {
	
	if ($name =~ /^[A-Z]/)
	{
	    my $nm = lc($name);
	    my $file;
	    for $file (sort keys %{$search{$name}})
	    {
		$search{$nm}{$file} = "";
	    }
	    $search{$nm}{""} .= " ".$name;
	    next;
	}
	
	if ($name =~ /^([a-z])/)
	{
	    my $nm = uc($1);
	    if ($out{$list}{info} !~ /\"\#$nm\"/)
	    {
		$out{$list}{info} .= $n.'<a href="#'.$nm.'">['.$nm.']</a>';
		$out{$list}{body} .= '<a name="'.$nm.'"></a>'.$n;
	    }
	}
	
	$out{$list}{body} .= "<dt>         ".$name.$search{$name}{""};
	$out{$list}{body} .= $n."</dt><dd>".$n;
	my $file; for $file (sort keys %{ $search{$name} })
	{
	    next if not length $file;
	    $out{$list}{body} .= ' <a href="'.$file.'">'.$n;
	    $file =~ s:[.]\w+$::;
	    $out{$list}{body} .= '          '.$file.' </a> '.$br.$n;
	}
    }
    
    $out{$list}{body} .= "</dl>".$n;
    $list = "";
}

#&search_index_words();
#&search_index_body("search.html");

# ------------------------------------------------------- MACRO CATEGORIES ---

sub macro_categories
{
    my $file;
    for $file (keys %out) # foreach $file of type "macro"
    {
	next if not $out{$file}{type} eq "macro";
	print STDERR "," if not $o{quiet};
	
	# category checks
	if (length $out{$file}{category})
	{
	    my $also = $out{$file}{category};
	    if (length $def{add}{$file})
	    {   $also = "" if $also eq $out{$file}{into};
		$also .= "," if length $also;
		$also .= $def{add}{$file}; 
	    }elsif (length $def{add}{ $out{$file}{name} })
	    {   $also = "" if $also eq $out{$file}{into};
		$also .= "," if length $also;
		$also .= $def{add}{ $out{$file}{name} }; 
	    }elsif (length $def{add}{ $out{$file}{orig} })
	    {   $also = "" if $also eq $out{$file}{into};
		$also .= "," if length $also;
		$also .= $def{add}{ $out{$file}{orig} }; 
	    }
	    $out{$file}{obsoleted} = $also if $also =~ /obsoleted/;
	    $also =~ s/\([^()]*\)//g;
	    $also =~ s/,.*//;
	    $also =~ s:^\s*::; $also =~ s:\s*$::; $also =~ s:\s+:_:g;
	    $out{$file}{also} = $also if length $also;
	}
	
	if (length $out{$file}{obsoleted})
	{
	    $out{$file}{also} = $out{$file}{into} if length $out{$file}{into};
	    $out{$file}{into} = "obsoleted";
	}
    }   print STDERR $n if not $o{quiet};
}

#&macro_categories();

# ------------------------------------------------------- make %index / %sect
# fill the %index and %sect list,  these are inverted indexes 
# that are used later to speed up (ordered) processing

sub categories_index_pages
{
    my $file;
    for $file (keys %out)
    {
	next if not $out{$file}{type} eq "macro";
	# create virtual index page if mentioned as $into
	my $name = $out{$file}{into};
	$name = "local" if not length $name;
	if (not exists $out{"$name.html"} and 
	    not exists $out{$name."/index.html"})
	{
	    $out{"$name.html"}{name} = "$name";
	    $out{"$name.html"}{file} = "$name.html";
	    $out{"$name.html"}{type} = "index";
	    $out{"$name.html"}{sect} = "virt";
	    $out{"$name.html"}{sort} = "ac-macros";
	    $out{"$name.html"}{title} = "$name";
	    $out{"$name.html"}{chapter} = "$name macros";
	}
    }
}

my %sect; # $sect{$ion}{$name} -> $file
sub categories_fill_sect
{
    my %index;

    my $file;
    for $file (keys %out)
    {
	next if $out{$file}{type} ne "index";
	$sect{ $out{$file}{sect} }{ $out{$file}{name} } = $file;
	print " ", $out{$file}{sect} ," ", $out{$file}{name} ," ", $file,$n;
    }
}

sub index_categories()
{
    &categories_index_pages();
    &categories_fill_sect();
}

#&fill_index_sect();

# ------------------------------------------------------ MAKE INDEX PAGES ---

# set $file{navi} marker
sub mark_navi_pages
{
    my $file;
    for $file (sort keys %out)
    {
	# which file goes listed on navi bar
	my $name;
	for $name (qw/index docus/)
	{	
	    next if not $out{$file}{type} eq $name;
	    $out{$file}{navi} = "$name" if not length $out{$file}{navi}; 
	}
    }
}
#&mark_navi_pages();

# creates $file{intohtmlindex} - the main section
sub gen_intohtmlindex # $file (the index page)
{
    my ($list,$xxx) = @_;
    my $T = "  <ul>\n";
    my $file;
    for $file (sort keys %out)
    {
	next if not $out{$file}{type} eq "macro";
	next if not $out{$file}{into} eq $out{$list}{name};
	my $m4file = $out{$file}{orig}; $m4file =~ s/\.\w+$//; # chop ext
	my $htfile = $file;
	if ($list =~ m:/:)
	{
	    $m4file =~ s/.*\///;    # basename
	    $htfile =~ s/.*\///;    # basename
	}
	$T .= "    <li> <a href=\"".$htfile."\">".$m4file."</a></li>\n";
    }
    $T .= "  </ul>\n";
    $out{$list}{intohtmlindex} = $T;
}

# creates $file{alsohtmlindex} - the "also" sections...
sub gen_alsohtmlindex # $file (the index page)
{
    my ($list,$xxx) = @_;
    my $T = "";
    my  $ion; my $last = "";
    for $ion (sort keys %sect) 
    {
	my $name;
	for $name (sort keys %{$sect{$ion}})
	{
	    next if $name eq $out{$list}{name}; # already in main section

	    my $file;
	    for $file (sort keys %out)
	    {
		next if not $out{$file}{type} eq "macro";
		next if not $out{$file}{into} eq $name;
		next if not $out{$file}{also} eq $out{$list}{name};
		if ($last ne $name)
		{
		    $last = $name;
		    $T .= "<h2>";
		    $T .= $out{$sect{$ion}{$name}}{chapter};
		    $T .= ":"."</h2>\n  <ul>\n";
		}
		
		my $N = $file; $N =~ s/.*\///;
		my $htfile = (($list =~ m:/:)?"../":"").$file;
		$T .= "    <li> <a href=\"".$htfile."\">".$N."</a>\n";
	    }
	    $T .= "  </ul>\n" if ($last eq $name);
	}
    }
    $out{$list}{alsohtmlindex} = $T;
}

# create $file{infohtmlindex}
sub gen_infohtmlindex
{
    my ($list,$xxx) = @_;
    my $T;
    if (exists $def{info}{ $out{$list}{name} })
    {
	my $name = $out{$list}{name};
	$T .= '<p>&nbsp;</p> '.$n
	    .'<p align="center">'.$def{info}{$name}.'</p>'
	    .$n.'<p align="center">......................</p>'.$n;
    }
    if (exists $def{gnus}{ $out{$list}{name} })
    {
	my $name = $out{$list}{name};
	$T .= '<p>&nbsp;</p>' if not length $T;
	$T .= $n
	    .'<p align="center">- mirrored -</p>'.$n.'<P>'.$n
	    .'The macros in the main section of this category are mirrored'.$n
	    .'from the'
	    .' <a href="'.$def{gnu}{webref}.'">'.$def{gnu}{archive}.'</a>'.$n
	    .' category of the same name.</P><p align="center">'.$n
	    .'<a href="'.$def{gnu}{webref}.$def{gnus}{$name}.'">';
	$name =~ s/_/&nbsp;/g;
	$T .= $def{gnu}{archive};
	$T .= "'s&nbsp; <b>".$name.'</b></a>'.$n.'</p>';
	$T .= '<p>&nbsp;</p><p align="center"><small>'
	    .'<i>just the macros below are additional</i>'
	    .'</p>'.$n if length $out{$list}{alsohtmlindex};
    }
    $out{$list}{infohtmlindex} = $T;
}

# creates $file{body} foreach $file{type}="index"
sub gen_index_pages
{
    my $list;
    for $list (sort keys %out)
    {
	next if not $out{$list}{type} eq "index"; # check "index" pages
	&gen_intohtmlindex($list);
	&gen_alsohtmlindex($list);
	$out{$list}{body} .= $out{$list}{intohtmlindex};
	$out{$list}{body} .= $out{$list}{alsohtmlindex};

	&gen_infohtmlindex($list);
	$out{$list}{info} .= $out{$list}{infohtmlindex};
	if (length $out{$list}{info} > 10)
	{
	    $out{$list}{body} .= "<!--spacer-->".$n."<pre>";
	    $out{$list}{body} .= " " x 50;
	    $out{$list}{body} .= "</pre>".$n;
	}
    }
}

sub gen_index_frontpage
{    
    my $list = $topx; # the frontpage (only "gnus" sections are allowed here)
    {   my  $ion = "gnus";
	for $ion (qw/gnu gnus/)
	{
	    my $name;
	    for $name (sort keys %{$sect{$ion}})
	    {
		$out{$list}{body} .= "<h2>"
		    .$out{ $sect{$ion}{$name} }{chapter}.":"."</h2>\n";
		$out{$list}{body} .= "  <ul>\n";

		my $file;
		for $file (sort keys %out)
		{
		    next if not $out{$file}{type} eq "macro";
		    # print "(".$out{$file}{into}.")";
		    next if not $out{$file}{into} eq $name;
		    my $N = $file; $N =~ s/.*\///;
		    $out{$list}{body} .= "    <li> <a href=\""
			.$file."\">".$N."</a>\n";
		}
		$out{$list}{body} .= "  </ul>\n";
	    }
	}
    }
    # $out{$list}{name} = "$name";
    $out{$list}{file} = "$list";
    $out{$list}{type} = "index";
    $out{$list}{navi} = "frontpage";
    $out{$list}{sect} = "virt";
    $out{$list}{sort} = "ac-macros";
}

#&gen_index_pages();
#&gen_index_frontpage();

# ---------------------------------------------------------- INDEX TO HTML ---

sub add_intro_text_for_index_pages
{
    my $file;
    for $file (keys %out)
    {
	next if not exists $def{intro}{$file};
	$out{$file}{info} = $def{intro}{$file};
    }
}

my @indexlist;
sub build_sorted_list_of_index_pages
{
    my  $ion;
    for $ion (sort keys %sect)
    {
	my $name;
	for $name (sort keys %{$sect{$ion}})
	{
	    # print ":::",$name, " :: ", $sect{$ion}{$name},$n;
	    push @indexlist, $sect{$ion}{$name};
	}
    }
}

my %cat;
sub  build_sorted_list_of_extra_pages
{
    my $file;
    for $file (sort keys %out)
    {
	next if $out{$file}{type} ne "docus";
	$cat{ $out{$file}{sort} } = "1";
    }
}

sub build_indexcatpages
{
    &add_intro_text_for_index_pages();
    &build_sorted_list_of_index_pages();
    &build_sorted_list_of_extra_pages();
}

sub navipage_logoblurb # --> $text
{
    return "<!--logo:-->".$n
	."<center>\n".$def{html}{logo}."</center>".$n	."<!--logo/-->".$n
	.'<hr />'.$n
	."<!--site:-->".$n
	.$def{html}{sfnetlinks} ."<!--site/-->".$n
	.'<hr />'.$n;
}

sub navipage_copyrightblurb # --> $text
{
    my $T = "";
    my $date = scalar localtime();
    $T.='<small><i>generated...</i>'.$br.'<center>'.$date.'</center></small>';
    $date =~ s/.*\s//;
    $T.=$br.'<b><a href="copyright.html">(C) '.$date.'</a> ';
    $T.=' <a href="mailto:guidod[@]gmx.de">guidod</a></b>';
    return $T;
}

sub navipage_macrodirs # --> $text
{
    my $T = "";
    ####### my $back = ""; $back = "../" if $file =~ m:/:; ##############
    $T.="<!--dirs:-->\n";
    $T.='<a href="../'.$topx.'">Macro Index</a>'.$br;
    my $name;
    foreach $name (@indexlist)
    {
	$T.= '- <a href="../'.$name.'">'.$out{$name}{title}.'</a>'.$br;
    }
    $T.="<!--dirs/-->".$n.'<hr />'.$n;
    return $T;
}

sub navipage_doculist # --> $text
{
    my $T = "";
    $T.="<!--docs:-->".$n;
    my $category; 
    for $category (sort keys %cat)
    {
	next if -d $category or $category eq "ac-macros";
	print STDERR "<$category>";
	my $name;
	foreach $name (sort keys %out)
	{
	    next if not $out{$name}{type} eq "docus";
	    next if not $out{$name}{sort} eq $category;
	    $T.='- ' if not $out{$name}{title} eq $out{$name}{category};
	    $T.='<a href="../'.$name.'">'.$out{$name}{title}.'</a>';
	    $T.=$br;
	}
    }
    $T.="<!--docs/-->".$n.'<hr />'.$n;
    return $T;
}

# create navi-column ...................................................
# note that we create the navi-column's linkends as if the current page
# is in a first-level subdirectory, so that each of the sister pages
# is referenced via "../category/index.html"
sub build_navibodypage() # $file ( the $navibodypage )
{
    my ($file,$xxx) = @_;
    my $T = "";
    $T.= navipage_logoblurb();
    $T.= navipage_macrodirs();
    $T.= navipage_doculist();
    $T.= navipage_copyrightblurb();
    $out{$file}{html} = $T;
}

my $navibodypage = "body.htm_";
sub build_indexhtml
{
    &build_indexcatpages();
    &build_navibodypage($navibodypage);
}
    
#&build_indexhtml();


# -------------------------------------------------- ( GENERATE OUTPUT PAGES )

# sets $out{$file}{htmltitle}
sub generate_htmltitle
{
    my ($file,$xxx) = @_;
    $out{$file}{htmltitle} = $def{short}{title}.' - '.$out{$file}{title};
}

# sets $out{$file}{htmlmetablock}
sub generate_htmlmetablock
{
    my ($file,$xxx) = @_;
    my $T = "";
    {
	$T.='<meta name="DC.Publisher"  content="'
	    .$def{short}{title}.'" />'.$n;
    }
    if ($out{$file}{type} eq "index")
    {   $T.='<meta name="DC.Type"       content="Collection"'.' />'.$n;  
	$T.='<meta name="DC.Title"      content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{title}.'" />'.$n;
	$T.='<meta name="revisit-after" content="30 days"'.' />'.$n
	    unless $out{$file}{name} eq "search";
    }
    elsif ($out{$file}{type} eq "macro")
    {   $T.='<meta name="DC.Type"       content="Software"'.' />'.$n;  
	$T.='<meta name="DC.Title"      content="'
	    .$def{short}{title}.' Autoconf Extension '
	    .$out{$file}{title}.'" />'.$n;
	$T.='<meta name="DC.Relation"   content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{into}.'" />'.$n;
	$T.='<meta name="...Relation"   content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{also}.'" />'.$n if length $out{$file}{also};
    }
    elsif ($out{$file}{type} eq "docus")
    {   $T.='<meta name="DC.Type"       content="Text"'.' />'.$n;  }

    # ----------------------------------------------------------------
    if (length $out{$file}{author})
    {
	my $name = $out{$file}{author};
	$name =~ s/[^\w\s.-]/:/gs; $name =~ s/\w*:.*//gs; 
	$name =~ s/^\s+//gs;       $name =~ s/\s+$//gs;  $name =~ s/\s+/ /gs;
	$T.='<meta name="DC.Creator"    content="'.$name.'" />'.$n
	    if length $name;
    }
    if (length $out{$file}{orig})
    {
	my @A = stat $out{$file}{orig};
	@A = gmtime $A[9]; # $mtime
	$T.='<meta name="DC.Date"       content="'
	    .(1900+$A[5])."-".(1+$A[4])."-".(1+$A[3]).'" />'.$n;
	$T.='<meta name="DC.Source"     content="file:'
	    .$out{$file}{orig}.'" />'.$n;
    }
    if (length $out{$file}{words})
    {
	$T.='<meta name="words"         content="'
	    .$out{$file}{words}.'" />'.$n;
    }
    $out{$file}{htmlmetablock} = $T;
}

# sets $out{$file}{htmlstyle}
sub generate_htmlstyle
{
    my ($file,$xxx) = @_;
    $out{$file}{htmlstyle} = $def{html}{style};
    my $s;
    for $s (keys %style) {
	next if not $style{$s}{used};
	next if not $style{$s}{head};
	$out{$file}{htmlstyle} .= $n."  ".$style{$s}{head};
    }
}

$style{secttable}{mark} = 'table width="100%" cellpadding="0" cellspacing="0"';
$style{sectstart}{mark} = 'tr';
$style{sectblock}{mark} = 'td';
$style{sectversion}{mark} = 'td align="right"';
$style{'secttitle'}{mark} = "h2";
$style{'secttitle size="small"'}{mark} = "h3";

sub generate_sectpart # $file
{
    my ($file,$xxx) = @_;
    my $T = "";
    $T.="<secttable><sectstart>".'<sectblock>'.$n;
    if (length $out{$file}{chapter}) {
	$T.="<secttitle>".$out{$file}{chapter}."</secttitle>";
    }else{
	$T.='<secttitle size="small">The Autoconf Macro Archive</secttitle>';
    }
    $T.= $n.'</sectblock><sectversion>'.$n;
    $T.= $n.$def{html}{sfversion}.$n."<sectbutton />".$n;
    $T.='</sectversion>'.'</sectstart></secttable>'.$n;
    $out{$file}{sectpart} = $T;

    $style{sectbutton}{mark} = $def{html}{sfbutton};
}

$style{navipagetable}{mark} = 'table width="100%"';
$style{navipagestart}{mark} = 'tr valign="top"';
$style{navipageindex}{mark} = 'td width="145" bgcolor="#E0E0FF"';
$style{navipageblock}{mark} = 'td';
$style{infotable}{mark} = 'table width="100%"';
$style{infostart}{mark} = 'tr valign="top"';
$style{infoindex}{mark} = 'td';
$style{infoblurb}{mark} = 'td';

sub generate_htmlbodypart
{
    # PREPEND navi block table and put the $file{body} on the right side
    my ($file,$xxx) = @_;
    generate_sectpart($file);
    my $T = "<navipagetable><navipagestart><navipageindex>".$n;
    my $Q = $out{$navibodypage}{html};
    $Q =~ s{^(-* *)(<a\s\Qhref=\"../$file\"\E.*)}{$1<b>$2</b>}mg;
    $Q =~ s{(\shref=\")\Q../\E}{$1}sg if ($file !~ m:/:);
    $T.= $Q;
    $T.=$n.'</navipageindex><navipageblock>'.$n;
    $T.= $out{$file}{sectpart};
    $T.="<infotable><infostart><infoindex>" if length $out{$file}{info};
    $T.='<!--body:-->'.$n;
    $T.= $out{$file}{body}; # <------------- body text
    $T.='<!--body:-->'.$n;
    $T.="</infoindex><infoblurb>".$n            if length $out{$file}{info};
    $T.=$out{$file}{info}                       if length $out{$file}{info};
    $T.="</infoblurb></infostart></infotable>"  if length $out{$file}{info};
    $T.='</navipageblock></navipagestart></navipagetable>'.$n;
    $out{$file}{htmlbodypart} = $T;
}

# sets $out{$file}{htmlbody} and $out{$file}{htmlbodyinfo}
sub generate_htmlbody
{
    my ($file,$xxx) = @_;

    if (not length $out{$file}{navi})
    {
	$out{$file}{htmlbody} = expand_html_style($out{$file}{body});
	$out{$file}{htmlbodyinfo} = " +".(length $out{$file}{htmlbody});
	return; # only $file{body} if not navi <-----------------
    }

    # PREPEND navi block table and put the $file{body} on the right side
    $out{$file}{htmlbodyinfo} .= "[".$out{$file}{navi}."]";
    &generate_htmlbodypart($file);
    $out{$file}{htmlbody} = expand_html_style($out{$file}{htmlbodypart});
    $out{$file}{htmlbodyinfo} .= " *".(length $out{$file}{htmlbody});
}

# sets $out{$file}{htmlpage} and $out{$file}{htmlpageinfo}
sub generate_htmlpage
{
    # we assume a $out{$file}{body} to exist, we generate 
    # a navigation bar to the left and place appropriate
    # meta data according to $out{$file}{title/chapter/info}
    my ($file,$xxx) = @_;
    my $T = "";

    &generate_htmltitle($file);
    &generate_htmlmetablock($file);
    &generate_htmlbody($file);
    &generate_htmlstyle($file); # so expand_html_style can work
    $T.="<html><head>";
    $T.="<title>".$out{$file}{htmltitle}."</title>".$n
	if length $out{$file}{htmltitle};
    $T.=$out{$file}{htmlmetablock};
    $T.="<style>\n".$out{$file}{htmlstyle}."</style>\n"
	if length   $out{$file}{htmlstyle};
    $T.="</head><body>".$n; $T.=$out{$file}{htmlbody};
    $T.='</body></html>'.$n;
    $out{$file}{htmlpage} = $T;
    $out{$file}{htmlpageinfo} = $out{$file}{htmlbodyinfo};
}

sub generate_xmlpage
{
    # we assume a $out{$file}{body} to exist, we generate 
    # a navigation bar to the left and place appropriate
    # meta data according to $out{$file}{title/chapter/info}
    my ($file,$xxx) = @_;
    my $T = '<!DOCTYPE index PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"'.$n;
    $T .= '   "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd" ['.$n;
    $T .= '<!ENTITY nbsp "&#x00A0;">'.$n;
    $T .= ']>'.$n."<index>".$n;

    &generate_htmltitle($file);
    &generate_htmlbodypart($file);
    &generate_htmlstyle($file);
    $T.="<macroindexpage><macroindexhead>";
    $T.="<title>".$out{$file}{htmltitle}."</title>".$n
	if length $out{$file}{htmltitle};
    $T.="<style>\n".$out{$file}{htmlstyle}."</style>\n"
	if length   $out{$file}{htmlstyle};
    $T.="</macroindexhead><macroindexbody>".$n; $T.=$out{$file}{htmlbodypart};
    $T.='</macroindexbody></macroindexpage>'.$n;
    $T.='</index>'.$n;
    $out{$file}{xmlpage} = $T;
}

sub generate_output
{
    my $testfile = $o{test};
    $testfile = "test.out" if length $testfile < 2;
    unlink $testfile if length $o{test};

    my $file;
    foreach $file (sort keys %out)                   # foreach output file...
    {
	print STDERR $out{$file}{type}," ",$file, ".."; 
	
	my $F;
	if (length $o{test})
	{
	    $F = $testfile;
	    if (not open F, ">>$F")
	    {
		print "(could not open: $!)\n";
		next; # OOPS
	    }
	}else
	{
	    $F = $file;
	    if (not open F, ">$F")
	    {
		print "(could not open: $!)\n";
		next; # OOPS
	    }
	}
    
	if (length $out{$file}{html})
	{ # pre-formatted html, e.g. macro2html result
	    print F $out{$file}{html};
	    close F; 
	    print STDERR " -",length $out{$file}{html} unless $o{test}; 
	    print STDERR $n;
	}else
	{ # generate htmlpage from $file{body} and perhaps a navi block
	    print STDERR ".." unless $o{test}; # ____________
	    &generate_htmlpage($file);
	    print F $out{$file}{htmlpage}; close F;
	    print STDERR $out{$file}{htmlpageinfo} unless $o{test};
	    &generate_xmlpage($file);
	    $F =~ s/[.]html$/.xml/;
	    if ($o{xml} and length $out{$file}{xmlpage} and open F, ">$F") {
		print F $out{$file}{xmlpage}; close F;
		print STDERR "..";
	    }
	    print STDERR $n;
	}
    };
}

#&generate_output();

1;

__DATA__
INFO obsoleted
    Just Obsolete Macros!
